@@ -9,7 +9,7 @@
 #include "putty.h"
 #include "misc.h"
 #include "ssh.h"
-#include "sshchan.h"
+#include "ssh/channel.h"
 #include "psocks.h"
 
 /*
@@ -94,8 +94,7 @@ static const SshChannelVtable psocks_scvt = {
 
 static void psocks_plug_log(Plug *p, PlugLogType type, SockAddr *addr,
                             int port, const char *error_msg, int error_code);
-static void psocks_plug_closing(Plug *p, const char *error_msg,
-                                int error_code, bool calling_back);
+static void psocks_plug_closing(Plug *p, PlugCloseType, const char *error_msg);
 static void psocks_plug_receive(Plug *p, int urgent,
                                 const char *data, size_t len);
 static void psocks_plug_sent(Plug *p, size_t bufsize);
@@ -354,8 +353,8 @@ static void psocks_plug_log(Plug *plug, PlugLogType type, SockAddr *addr,
     };
 }
 
-static void psocks_plug_closing(Plug *plug, const char *error_msg,
-                                int error_code, bool calling_back)
+static void psocks_plug_closing(Plug *plug, PlugCloseType type,
+                                const char *error_msg)
 {
     psocks_connection *conn = container_of(plug, psocks_connection, plug);
     if (conn->connecting) {
@@ -523,8 +522,8 @@ void psocks_start(psocks_state *ps)
  * Some stubs that are needed to link against PuTTY modules.
  */
 
-int verify_host_key(const char *hostname, int port,
-                    const char *keytype, const char *key)
+int check_stored_host_key(const char *hostname, int port,
+                          const char *keytype, const char *key)
 {
     unreachable("host keys not handled in this tool");
 }
